;  Compile: bigloo -o scd scd-factorial.scm gtk.o  -lgtk-server

(module test (main start-here)
  (include "macro-def.sch")
  (import (gtkserver_gtk "gtk.scm")))

(define (start-here argv)
  (gtk-init)
  (let* ( (win (gtk-window-new 0))
          (hbx (gtk-hbox-new #f 0))
          (vbx (gtk-vbox-new #f 0))
          (entr (gtk-entry-new))
          (ext (gtk-button-new-with-label "exit"))
          (fact (gtk-button-new-with-label "!"))
          (data #f) )
     
      (gtk-box-pack-start hbx entr 1 1 0)
      (gtk-box-pack-start hbx fact 1 1 0)
     
      (gtk-box-pack-start vbx  ext 1 1 0)
      (gtk-box-pack-start vbx hbx 1 1 0)
     
      (gtk-container-add  win vbx)
      (gtk-widget-show-all win)
      (play-with event until (equal? event ext) thus
         [ fact
             (set! data (string->number (gtk-entry-get-text entr)))
             (when data 
                 (gtk-entry-set-text entr 
                       (number->string (factorial data)  )))  ]
) )   )

(define (factorial n)
  (if (< n 1) 1
      (* n (factorial (- n 1)))))
      

;;;;;;;;;;;;;;;;;;;  Some action} ;;;;;;;;;;;;;;;;;;
;;
;; Let us see how we add an action button to a GUI application. The action 
;; button is supposed to read a number from a data entry, calculate its 
;; factorial, and return the result to the data entry. Therefore, if you 
;; want to accomplish these objectives, you must create a data entry. 
;; Then you will have three components on your window:
;;
;;   (entr (gtk-entry-new))                            ;;The data entry field
;;   (ext (gtk-button-new-with-label \texttt{"exit"})) ;;The exit button
;;   (fact (gtk-button-new-with-label \texttt{"!"}))   ;;The factorial button
;;
;; As for the layout, the idea is to have the factorial button situated
;; on the right hand side of the data entry field. To accomplish this kind of
;; layout, you must create a horizontal box (hbx) and a vertical box (vbx). The 
;; data entry (entr) and the factorial button (represented by an exclamation mark) 
;; will go into hbx. When it comes to its turn, hbx will be stacked inside vbx.
;;
;;    (gtk-box-pack-start hbx entr 1 1 0)
;;    (gtk-box-pack-start hbx fact 1 1 0)
;;    (gtk-box-pack-start vbx  ext 1 1 0)
;;    (gtk-box-pack-start vbx hbx 1 1 0)
;;
;; The 'play-with' form has a sequence of cond-like conditions that represent 
;; activation of the action buttons. Here, there is only one action button.
;; Therefore, 'play-with' has only one condition.
;;
;; (play-with event until (equal? event ext) thus
;;    [ fact
;;       (set! data (string->number (gtk-entry-get-text entr)))
;;       (when data
;;          (gtk-entry-set-text entr
;;          (number->string (factorial data)  )))  ]
