;;
;; Compile with:
;;
;;;; bigloo -o sce-msg.exe sce-message-dialog.scm gtk.o -lgtk-server


(module test (main start)
  (include "macro-def.sch")
  (import (gtkserver_gtk "gtk.scm")))
    
(define (start arqv)
  (gtk-init)
  (let* ( (win (gtk-window-new 0)) 
          (dlg   (gtk-message-dialog-new win 0 1 5 "Bye!")) ;; 1- warning 5- ok/cancel
          (table  (gtk-table-new 30 30 1)) 
          (exit-button (gtk-button-new-with-label "Exit"))       
          (label (gtk-label-new "Bigloo running GTK") ) )
       
     (gtk-table-attach-defaults table exit-button 10 29 25 29) ;20 29 25 29)
     (gtk-table-attach-defaults table label 5 25 10 20)
    
     (gtk-container-add  win table)
     (gtk-window-set-title win "Demo with GTK-server lib")  
     (gtk-widget-show-all win)
     
     (do ([event (gtk-server-callback) 
                 (gtk-server-callback)] )
         ((equal? event exit-button) ; did you pressed the exit-button
             ;; If so, print the text and leave:
             (print (gtk-dialog-run dlg))
             ; If you press OK, the program prints -5
             ; If you press CANCEL, it prints -6
             ; YES -> -8, NO -> -9
         );close exit condition
      );closed do
     
  );close let-star 
);close define

;; In this program, you will put a dialog button to work. If the 
;; user presses the exit button, s/he will attribute  "button" to
;; event; this will make the clause (equal? event button) be true 
;; (or #t, in Scheme). The result is that the server will interrupt
;; the do loop:
;;
;;       (do ((event (gtk-server-callback) 
;;              (gtk-server-callback)) )
;;           ((equal? event button))
;;
;; The clause ((event (gtk-server-callback) (gtk-server-callback))) initializes
;; the variable 'event' to the value returned by (gtk-server-callback);
;; the next time that it goes through the do loop, the server updates
;; 'event' to a new value. Events produced by (gtk-server-callback) are
;; things like button pressed, or mouse clicked.
;;
;; Let us explain how the table layout used in this application works.
;; Tables are containers, i.e., entities that can hold GUI components
;; like buttons and data entries. You have seen two examples of containers
;; before: vbox and hbox. Using tables, it is possible to create a grid
;; in which you can place widgets. The function that creates a table
;; has three arguments:
;;
;; (gtk-table-new  30 ;;;  number of   rows
;;                 30 ;;;  number of columns
;;                 1 ) ;;;  homogeneous grid
;;
;; The first argument is the number of rows contained in the table, while the second
;; is the number of columns. The homogeneous argument shows how the table's boxes
;; are sized. If it is 1, the table's boxes are resized to the size of the largest
;; widget in the table. If homogeneous is 0, the size of a table's boxes is dictated
;; by the tallest widget within the same row, and the widest widget along the column.
;; 
;; The rows and columns are laid out from 0 to n, where n was the number specified
;; in the call to 'gtk-table-new'. So, if you specify rows = 3 and columns = 5,
;; the layout would look something like this:
;;
;;        0          1          2	   3	      4		 5
;;      0 +==========+==========+----------+----------+----------+
;;        ||         |         ||          |          |          |
;;      1 +==========+==========+----------+==========+==========+
;;        |          |          |          ||         |         ||
;;      2 +----------+----------+----------+==========+==========+
;;        |          |          |          |          |          |
;;      3 +----------+----------+----------+----------+----------+
;;
;; Notice that the coordinate system starts in the upper left hand corner. To place a widget
;; into the box that is at the upper left hand side corner, use the following  function:
;;
;;     (gtk-table-attach-defaults tbl exit-button 0 2  0  1)
;;
;; To place a widget in the other box, the command is
;;
;;     (gtk-table-attach-defaults tbl label-widget  1 2  3 5)
;;
;; You can create a message dialog using the command shown below.
;;
;;     (dlg   (gtk-message-dialog-new win 0 1 5 "Bye!"))
;;
;; The first argument (win) is the parent window. The second number code (third argument) 
;; indicates the kind of dialog. In the example, the code is 1, which indicates a 'warning' dialog. 
;; The possibilities are
;;
;;     0  Information
;;     1  Warning
;;     2  Question
;;     3  Error
;;
;; The third number code, that is 5 in the example, represents the
;; kind of buttons and their quantities. The list of codes is given below.
;;           
;;            0 has no button
;;            1 has an OK button
;;            2 has a CLOSE button
;;            3 has a CANCEL button
;;            4 has YES/NO buttons
;;            5 has OK/CANCELL buttons
;;
;; When the dialog ends, it returns a code that depends on the button pressed. The following table 
;; contains the answers given by all the message dialog buttons:
;;
;;    OK         --> -5
;;    CANCEL     --> -6
;;    YES        --> -8
;;    NO         --> -9
;;

