;; Compile with:
;;
;;	bigloo -o scf-edit scf-edit.scm gtk.o -lgtk-server
;;

(module test (main main)
  (include "macro-def.sch")
  (import (gtkserver_gtk "gtk.scm")))
    
(define (main arqv)
  (gtk-init)
  (let* ( (win (gtk-window-new 0)) 
          (tbl  (gtk-table-new 20 20 1))
          (startit (gtk-frame-new))
          (endit (gtk-frame-new))
          (ext (gtk-button-new-with-label "Exit")) 
          (sav (gtk-button-new-with-label "Save"))
          (lod (gtk-button-new-with-label "Load"))
          (en (gtk-entry-new))
          (hbx (gtk-hbox-new #f 0))
          (vbx (gtk-vbox-new #f 2)) 
          (sep  (gtk-hseparator-new)) 
          (sw (gtk-scrolled-window-new))
          (txt (gtk-text-buffer-new))
          (tv (gtk-text-view-new-with-buffer txt))
          (fname "")  
          (kw "")         )
    
     (pack-homogeneously hbx ext sav lod)
     (pack-pack vbx hbx sep en)	   
          
     (gtk-container-add sw tv)
     (gtk-table-attach-defaults tbl vbx 0 10 0 4 )
     (gtk-table-attach-defaults tbl sw 0 19 5 19)
	   
     (gtk-text-view-set-editable tv 1)
     (gtk-text-view-set-wrap-mode tv 2)
	  
     (gtk-container-add  win tbl)
     (gtk-window-set-title win "Simple text editor")  
     (gtk-widget-show-all win)
     
     (play-with event until (equal? event ext) thus
        [sav (set! fname (gtk-entry-get-text en))
             (set! fname (choose-file fname))
             (gtk-save-buffer txt fname startit endit)]
        [lod (set! fname (gtk-entry-get-text en))
             (gtk-text-buffer-set-text  txt "" -1)
             (for-each 
                 (lambda(x) 
                    (gtk-text-buffer-get-end-iter txt endit)
                    (gtk-text-buffer-insert txt endit x -1))
                 (gtk-load-lines fname) ) ]   )))

(define (choose-file x)
    (if (< (string-length x) 2) 
        "backup.txt" 
        x))

;; This example presents the difficult concept of iter.
;; I would like to thank Peter van Eerten for helping my
;; understanding of iters. Basically, an iter is a widget
;; that points to a text buffer. To retrieve the
;; text from a buffer, you need iters. Here is how to
;; get two brand new iters, one for finding where the
;; buffer starts, and the other for reaching its end.
;;
;;     (startit (gtk-frame-new)) 
;;     (endit   (gtk-frame-new)) 
;;
;; The GTK command below sets 'startit' and 'endit' to
;; the bounds of the buffer:
;;
;; (gtk-text-buffer-get-bounds txt startit endit) 
;;
;; Now that we know where the text starts and ends,
;; it is possible to retrieve the buffer contents:
;;
;;  (set! kw (gtk-text-buffer-get-text txt startit endit #t))
;;
;; As for the buffer, it is a string that stores every thing
;; that you type on the text-view. Here is how the buffer was
;; created in the program of the example:
;;
;; (txt (gtk-text-buffer-new)) 
;;
;; An example of how to insert text into the buffer:
;;
;;    (gtk-text-buffer-set-text  txt "" -1)
;;
;; To show the buffer, you need to associate it with
;; a text view:
;;
;; (tv (gtk-text-view-new-with-buffer txt))
;;
;; This snippet shows how to write programs with style. You know,
;; if your problem is to get there, any bike will do; but if you
;; want to get there with style, you will need something like a 
;; one-wheeled bike, or a very expensive aerodynamic three-wheeled
;; bike. If you take a careful look at the source of scf-edit.scm, 
;; you will notice that, in general, only functions that return 
;; something are in the definition section of the let*. Functions 
;; that do not return a value are like Gorgo's son: they are placed on
;; the top of  the let* -- such as (gtk-init) -- or behind it. For those
;; who do not know Gorgo's story, she was the beloved wife of Leonidas, 
;; king of Sparta. When the Persians invaded Greece, there was a shortage 
;; of steel to manufacture swords for everybody who wanted to fight. 
;; Therefore one of the swords was shorter than the others. The protocol 
;; required that the queen should be the person in charge of the distribution
;; of weapons.  Of course her son was in line, waiting for his sword. The 
;; Spartan youths were sure that the queen would not give the shorter sword 
;; to her son. But she did. The boy protested. "Mom, what can I do with this
;; short sword?" "Step forward", answered the queen. There are computer 
;; languages that are short of commands. They lack attribution, or side effect.
;; Programmers that use these languages step forward, and are able to write 
;; safer code than the others. Although Scheme does have attribution, you
;; should shun from  using this resource. 
;;
;; However, I still did not tell you the story of Gorgo's shield. Defeated soldiers
;; used to throw away their shields. This is reasonable, because the shield is heavy,
;; and it was difficult to flee from the battle field bearing a 80 pound shield. But
;; there were brave soldiers that kept their shield, in order to protect their fellow
;; combatants, and ensure an orderly retreat. If he died in battle, his companions 
;; would fetch his body on his shield. This behavior is what Gorgo expected from her son.
;; Thus, she recommended when she handed him the precious shield: Behind it, or on
;; top of it. Like Gorgo's son, functions that do not return values should be put
;; behind the let*, or on top of it.
;;
;; In this snippet, we did not use if, but cond. Notice that cond  gives a more elegant
;; organization to the button actions, spare parenthesis, and  makes the logical structure
;; of the program clear.
;;
;; I think that you have guessed what we are doing! Yes, we are writing a text editor. 
;; In this step, we have added a scrolling window, and organized the components in hboxed 
;; and vboxed. Let us start with the scrolling window. Instead of inserting tv directly 
;; into the component table tbl, we have inserted it into a scrolling window: 
;;
;;              (gtk-container-add sw tv). 
;;
;; The buttons were organized in a hbox:
;;
;;              (gtk-box-pack-start hbx ext 0 0 1)
;;	        (gtk-box-pack-start hbx sav 0 0 1)
;;	        (gtk-box-pack-start hbx lod 0 0 1)
;;
;; The entry and this hbox are put inside a vbox:
;;
;;  (gtk-box-pack-start vbx hbx 0 0 1)
;;  (gtk-box-pack-start vbx sep 0 0 1)
;;  (gtk-box-pack-start vbx en 0 1 1)
;;
;; The vbox and the scroll window are set inside a table:
;;
;;  (gtk-table-attach-defaults tbl vbx 0 10 0 4 )
;;  (gtk-table-attach-defaults tbl sw 0 19 5 19)
;;
;; Finally, the table is nailed to a window:
;; 
;; (gtk-container-add win tbl )
;; 
;; Besides these changes, the new program also checks for the existence of
;; reading files.
      
