;; Compile with:
;;
;;	bigloo -o scg-escher scg-escher.scm gtk.o -lgtk-server
;; 

;; This program was designed to exemplify the use of GTK-server's graphic functions.
;; When you enter the program, you will see one of Escher's famous designs (hopefully,
;; his state will not sue me :-). There are four copies of the design, one on the top
;; of the other. If you press the button yGreater, two copies will start to displace 
;; vertically in relation to the other two. Keep pressing yGreater until the pieces fit
;; into each other like puzzle pieces made from one of Escher's engravings. If you make
;; a mistake, and the displaced piece goes beyond the desired position, press the button
;; Neg once; then the next time you press the buttons yGreater or xGreater, the pieces
;; will  move in the opposite direction. Finally, displace the pieces in the x-direction
;; by pressing the xGreater button. The last step is to study the program, and learn how
;; it works. By the way, the only use for the small green rectangle that is above Escher's
;; horse is to teach me how to draw rectangles using the GTK-server.

(module cheval (main main)
  (include "macro-def.sch")
  (import (gtkserver_gtk "gtk.scm")))


(define (make-horse image gdkwin)
  (let* (  (xsize 400)
           (ysize 400)
           (rx 0)
           (ry 0)
           (PIX (gdk-pixmap-new gdkwin xsize ysize -1))
           (GC (gdk-gc-new PIX)) 
           (COLOR (gtk-frame-new)) )
     (lambda(dx dy)
       (g-object-unref PIX)
       (set! PIX (gdk-pixmap-new gdkwin xsize ysize -1))
       (gtk-image-set-from-pixmap image PIX)
       (gdk-color-parse "#00ff00" COLOR)
       (gdk-gc-set-rgb-fg-color GC COLOR)
       (set! rx (+ rx dx))
       (set! ry (+ ry dy))
       (for-each (lambda(x)
  	   (draw PIX GC x ysize 0)
           (draw PIX GC x (- ysize ry) 0) 
           ;; ======================
           (draw PIX GC x ysize rx)
           (draw PIX GC x (- ysize ry) rx) 
             ) cheval)  )))

(define  (draw pix gc xs ymax xp)
    (let ((a (car xs)) ) 
       (for-each 
           (lambda (b)
               (gdk-draw-line pix gc 
                      (+ xp (* 5 (car a))) 
	              (- ymax (* 5 (cadr a))) 
                      (+ xp (* 5 (car b))) 
                      (- ymax (* 5 (cadr b))))
               (set! a b)
             ) xs) ) )

               
(define (main args)
  (gtk-init)
  (let* ( (win (gtk-window-new 0))
          ;; Create widget to display image
          (image (gtk-image-new)) 
               
          ;; Create eventbox 
	  (EBOX (gtk-event-box-new))
	      
          (nnn 2)
               
          (ext  (gtk-button-new-with-label "Exit"))
          (neg  (gtk-button-new-with-label "Neg"))
          (xGt  (gtk-button-new-with-label "xGreater"))
          (yGt  (gtk-button-new-with-label "yGreater"))
	                   
          (hbx (gtk-hbox-new #f 0))
          (vbx (gtk-vbox-new #f 2))
          (horses #f)
          )

      (gtk-container-add EBOX image)
      (pack-homogeneously hbx ext neg yGt xGt)
      (pack-pack vbx hbx  EBOX)
      (gtk-container-add win vbx )
      (gtk-widget-show-all win) 
      (set! horses (make-horse image 
                      (gtk-widget-get-parent-window image) ))
      (gtk-window-set-resizable win #t)
      (gtk-window-set-default-size win 600 400)
	   
      (gtk-window-set-title win "Homage to Escher/Delahaye")
               
      (horses 0 0)

      (play-with event until (equal? event ext) thus
          [ neg (set! nnn (- nnn))]
          [xGt (horses nnn 0) ]
          [yGt (horses 0 nnn)]
 )     )   )
          
                  
(define cheval '(
    ((10 10)(8 12)(9 16)(12 17)(13 18)(14 20))
    ((13 18)(12 19)(9 21)(9 20)(10 19)(9 17)(7 20)(8 22)(12 22))
    ((12 20)(12 22)(13 20)(16 31)(18 31)(19 32) )
    ((16 31)(14 31 )(14 32) )

    ((14 31)(10 30)(12 31)(10 32)(10 34)(11 34)(11 33)(10 33) )

    ((12 32)(13 31))

    ((10 34)(16 36) )
    ((16 35)(16 37)(18 35)(17 34)) ; ear
    ((17 36)(20 36)(22 32)(19 26) ) ; behind the ear

    ((20 36)(22 36)(22 34)(24 32)(24 30)
                 (19 26)  (18 23)(21 22)(21 24) ) ;back
    ((30 30)(34 31)(36 31)(33 26)(32 22)
         (28 22)(27 20)(29 17)(30 19)(29 20) ) ; wing

    ;Legs
    ((29 21)(32 19)(33 18)(32 17)(29 16)(28 12)(30 10)(21 4)(21 2))
    ((16 17)(16 16)(10 14) (10 12)(12 11)(10 10));front leg

    ((18 3)(19 6)(24 10)(24 12) (22 14)(22 16)(23 17) );belly
    ((22 16)(17 16)(16 17)(17 18) )

    ((21 21)(22 24)(30 30) ); upper wing
    ((24 24)(34 28))
    ((25 23)(33 26))
    ((25 21)(27 20));feathers
    ((23 21)(24 19))
    ((27 20)(22 19)(22 21))
    ((22 19)(21 20))

    ((13 34)(15 35)(16 34)(16 33))
    ((15 35)(15 34)(16 34)(15 34)(15 35)); eye
    ((24 12)(26 10)(19 5)(19 3))
    ((28 22)(25 22))
) )
