;;
;; Compile with:
;;
;;	bigloo -o scm-ide scm-ide.scm gtk.o -lgtk-server
;;

;; This program shows how to write a simple IDE for Bigloo.
;; It was kept simple, so you can understand it easily, and
;; add features of your own.
;;
;; 1- Type the source code on the text viewer. 
;;
;; 2- If you want to save your source code, write the name of the
;;    file on the main text entry, and press the button "Save". 
;;
;; 3- To load a file, write the file name on the main text entry, and 
;;    press "Load". 
;;
;; 4- If you want to browse your computer, in order to find a good 
;;    place to save your source file, press the button "Browse". 
;;    You can use the browsing widget to inspect the file tree of your
;;    computer. Type the path on the browser's text entry, and press "OK".
;;    The browser will be closed, and the path will be transfered to 
;;    the main data entry.
;; 5- To search a string, write the string that you want to search on
;;    the main text entry, and press search. If you want to find the
;;    next instance of the string, press NxtSrc.
;; 
;; 6- To check parentheses, select the expression, and press CheckPar.
;;    If there is unclosed parenthesis, the expression will become red.
;;    Correct the error, select the expression again, and press CheckPar.
;;    After the correction, the selected expression will become black again.
;; 7- You can generate a Latex documentation of your program. To this
;;    end, it must start with comments, that will form the LaTeX introduction.
;;    A double semi-colon introduces a \section. Take a look at que.scx
;;    to see an example of source that you can latexify. The button
;;    "Set Title" reads the documentation title from the main text entry.
;;    The button "To LaTeX" taks a file name from the main text entry,
;;    add the extension ".tex" to it, and generates a LaTeX documentation.

(module test (main main)
  (include "macro-def.sch")
  (import (gtkserver_gtk "gtk.scm")))
    
(define (main arqv)
  (gtk-init)
  (let* ( (win::string (gtk-window-new 0))
          (_ (gtk-window-set-default-size win 600 480))
          (fileWin (gtk-window-new 0)) 
          (_ (gtk-window-set-default-size fileWin 400 200))
          (fileTable (gtk-table-new 10 20 0))
          (tbl::string  (gtk-table-new 20 20 1))
          (fromit (gtk-frame-new))
          (startit (gtk-frame-new))
          (endit   (gtk-frame-new))
         
          (mrk #f)
          (ext::string  (gtk-button-new-with-label "Exit")) 
          (sav::string  (gtk-button-new-with-label "Save"))
          (lod::string  (gtk-button-new-with-label "Load"))
          (brs (gtk-button-new-with-label "Browse"))
          (ok  (gtk-button-new-with-label "Ok"))
          
          (src (gtk-button-new-with-label "Search"))
          (nsr (gtk-button-new-with-label "NxtSrc"))
          (par (gtk-button-new-with-label "((()))"))
          (sTt        (gtk-button-new-with-label "Title"))
          (ltx::string (gtk-button-new-with-label "LaTeX"))
          
          (enFile (gtk-entry-new))
          (filsel (gtk-file-chooser-widget-new 0))
          (fileFilter (gtk-file-filter-new))
          (hfx (gtk-hbox-new #t 10))
         
          (en (gtk-entry-new))
          
          (hbx::string (gtk-hbox-new #t 4))
          (vbx::string (gtk-vbox-new #f 2)) 
          (sep  (gtk-hseparator-new)) 
          (sw (gtk-scrolled-window-new))
          (txt (gtk-text-buffer-new))
          (tv (gtk-text-view-new-with-buffer txt))
          (docTitle "Program documentation")
          (fname "a") 
          (browseName "c:/")
          (search-string "")
          (kw "")
          (fnd #f) )
        
     (pack-homogeneously hbx ext lod sav brs src nsr par sTt ltx)
     (pack-pack vbx hbx en sep)
     
     (gtk-container-add sw tv)
     (gtk-table-attach-defaults tbl vbx 1 19 0 4 )
     (gtk-table-attach-defaults tbl sw 1 20 5 19)
	   
     (gtk-text-view-set-editable tv 1)
     (gtk-text-view-set-wrap-mode tv 2)
	  
     (gtk-container-add  win tbl)
     (gtk-window-set-title win "Simple IDE")  
      
     (gtk-widget-show-all win)
     
     (gtk-box-pack-start hfx enFile 0 0 1)
     (gtk-box-pack-start hfx ok 0 0 1) 
     (gtk-table-attach-defaults fileTable hfx 0 10 0 1)
     (gtk-table-attach-defaults fileTable filsel 0 19 2 10)
     (gtk-container-add fileWin fileTable)
     
     (gtk-file-filter-set-name fileFilter  "all")
     (gtk-file-filter-add-pattern fileFilter "*.*")
     (gtk-file-chooser-add-filter filsel fileFilter)
     
     (gtk-text-buffer-create-tag txt "fstcolor" "foreground" "red")
     
     (do ((event (gtk-server-callback) 
	         (gtk-server-callback)) )
        ((equal? event ext)) 
        (cond ( (equal? event sav)
                
                (set! fname (gtk-entry-get-text en))
                                      
                (set! fname (if (< (string-length fname) 2) 
                                    "backup.txt" fname))
                (gtk-save-buffer txt fname startit endit))
	        
	      ( (equal? event lod)
                (set! fname (gtk-entry-get-text en))
                (gtk-text-buffer-set-text  txt "" -1)
		(let again ( (xs (gtk-load-lines fname)))
		      (when (not (null? xs))
                          (gtk-text-buffer-get-end-iter txt endit)
                          (gtk-text-buffer-insert txt endit  (car xs)  -1) 
		          (again (cdr xs)))))
              ( (equal? event sTt)
                (set! docTitle (gtk-entry-get-text en))
               
                (when (< (string-length docTitle) 2)
                      (set! docTitle "Program Documentation")))
              ( (equal? event ltx)
	        (set! fname (gtk-entry-get-text en))
	                             
	        (set! fname (if (< (string-length fname) 2) 
	                                          "backup" fname))
	      	(gtk-text-buffer-get-bounds txt startit endit)
	      	(set! kw (gtk-text-buffer-get txt startit endit #t) )
	        (latexIt kw (string-append fname ".tex") docTitle))
              ( (equal? event ok)
                (set! fname (gtk-entry-get-text enFile))
                
                (set!  browseName (gtk-file-chooser-get-filename filsel))
                   
                (string-replace! browseName #\\ #\/)
                
                (print browseName) (flush-output-port (current-output-port))
                
                (gtk-entry-set-text en (string-append browseName fname))
                                 
                (gtk-widget-hide fileWin))
              ( (equal? event brs)
                (gtk-widget-show-all fileWin) 
                (gtk-file-chooser-set-filename filsel browseName) )
	      ( (equal? event src) (first-search en txt fromit startit endit))
	      ( (and (equal? event nsr) mrk) (next-search en txt mrk fromit startit endit))
	      ( (equal? event par)
	       (set! fnd (gtk-text-buffer-get-selection-bounds txt startit endit))
	       (cond (   fnd 
	                 (set! kw (gtk-text-buffer-get-text txt startit endit #t))
	                 (gtk-text-buffer-remove-tag-by-name txt "fstcolor" startit endit)
	                 (cond ( (check-parentheses kw)
	                        
	                         (gtk-entry-set-text en "That is great!")
	                       )
	                       ( else (gtk-entry-set-text en "Something is rotten!")
	                              (gtk-text-buffer-apply-tag-by-name txt 
	                                              "fstcolor" startit endit) 
	                       )
	                 );close inner cond
	             );close selection found for checking parentheses
	             (  else ;no-selection for checking parentheses.
	                (gtk-entry-set-text en "Please, select the text you want to check")
	             );close selection not made condition 
	       );close selection cond
	    );close check parentheses condition
	);close outer cond
	        
      );close do 
   );close let*
);close define


(define (latexIt txt file-name docTitle)
       (let* ((pin (open-input-c-string txt))
              (cstring (read pin)))
        (with-input-from-string cstring
          (lambda()
             (let ( (resp '()))
	           (do ( (i (read-line) (read-line)))
	               ( (eof-object? i) 
	                 (save-latex (wrap-document docTitle (latexify resp)) 
	                             file-name))
	               (set! resp (cons i  resp))
	           )
	     )
	  )
	 )
	)
 )
  

(define (check-parentheses s)
   (let ( (zero-sum 0)
          (sz (- (string-length s) 1)  ) )
       (do ( (i 0 (+ i 1)))
           ( (> i sz) (equal? zero-sum 0))
           (when (equal? (string-ref s i) #\( )
                 (set! zero-sum (+ zero-sum 1)))
           (when (equal? (string-ref s i) #\) )
                 (set! zero-sum (- zero-sum 1))) ) ) )

(define (latexify xs)
  (app (get-doc #t xs '() '())))
  
(define (app xs)
   (cond ( (null? xs) '())
         (else (append (car xs) '(" ") (app (cdr xs)) )) ))

         
(define (latex-doc xs acc)
   (cond ( (null? xs) acc)
         (else (latex-doc (cdr xs) (cons (remove-comment (car xs)) acc) )
         )
   )
)

(define (is-comment? x)
   (cond ( (< (string-length x) 1) #f)
         ( (equal? (string-ref x 0) #\;) #t)
         (else #f)))
         
(define (not-comment? x)
   (cond ( (< (string-length x) 1) #t)
         ( (equal? (string-ref x 0) #\;) #f )
         (else #t)))

(define (add-block xs block)
  (cond ( (null? xs) block)
        (else (cons xs block))))

(define (get-doc comment xs acc blocks)
  (cond ( (and comment (null? xs)) (add-block acc blocks))
        ( (and (not comment) (null? xs)) (add-block (wrap-verbatim acc) blocks))
        ( (and comment (is-comment? (car xs))) 
          (get-doc comment (cdr xs) (cons  (remove-comment (car xs)) acc) blocks))
          
        ( (and comment (not-comment? (car xs)))
          (get-doc (not comment) (cdr xs) (list (car xs)) (add-block acc blocks)))
        ( (and (not comment) (not-comment? (car xs)))
          (get-doc comment (cdr xs) (add-block (car xs) acc) blocks))
        ( (and (not comment) (is-comment? (car xs)))
          (get-doc (not comment) (cdr xs) 
          (list (remove-comment (car xs))) 
          (cons (wrap-verbatim acc) blocks)))))


(define (remove-comment x)
   (cond ( (< (string-length x) 1) x)
         ( (section? x) (wrap-section x))
         ( (equal? (string-ref x 0) #\;) (string-set! x 0 #\space) x) 
         (else x) 
   )
)

(define (section? x)
   (if (< (string-length x) 4) #f
       (and (equal? (string-ref x 0) #\;) 
       (equal? (string-ref x 1) #\;)) ) )
       
(define (wrap-section x)
  (string-set! x 0 #\space)
  (string-set! x 1 #\space)
  (string-append "\\section{" x "}"))
      
(define (save-latex xs file-name)
  (call-with-output-file file-name 
     (lambda(out)
        (for-each (lambda(x) (display x out) (newline out)) xs)
        (close-output-port out)
     )     
  )
)

(define (wrap-verbatim xs)
   (append '("\\begin{verbatim}") xs '("\\end{verbatim}") ))

(define (wrap-document docTitle xs)
  (append (latex-header docTitle) xs '("\\end{document}")))

(define (latex-header title)
 (list "\\documentclass[12pt]{article}"
   (string-append "\\title{" title "}")
   "\\author{Ed Mort}"
   "\\date{}"
   "\\begin{document}"
   "\\maketitle"
))
