;;
;; Compile with:
;;
;;	bigloo -o sco-rect sco-rect.scm gtk.o -lgtk-server
;; 

;; This program was designed to exemplify the use of GTK-server's graphic functions.
;; When you enter the program, you will see one of Escher's famous designs (hopefully,
;; his state will not sue me :-). There are four copies of the design, one on the top
;; of the other. If you press the button yGreater, two copies will start to displace 
;; vertically in relation to the other two. Keep pressing yGreater until the pieces fit
;; into each other like puzzle pieces made from one of Escher's engravings. If you make
;; a mistake, and the displaced piece goes beyond the desired position, press the button
;; Neg once; then the next time you press the buttons yGreater or xGreater, the pieces
;; will  move in the opposite direction. Finally, displace the pieces in the x-direction
;; by pressing the xGreater button. The last step is to study the program, and learn how
;; it works. By the way, the only use for the small green rectangle that is above Escher's
;; horse is to teach me how to draw rectangles using the GTK-server.

(module cheval (main main)
  (include "macro-def.sch")
  (import (gtkserver_gtk "gtk.scm")))

(define (main args)
    (gtk-init)
    (let* ( (xmax 400)
            (ymax 400)
            (xp 0)
            (yp 0)
            (from '(0 0)) 
            
            (drL (lambda(p g  a b)
                   (let ((x1 (car a))
                        (y1 (cadr a))
                        (x2 (car b))
                        (y2 (cadr b)))
                        (gdk-draw-line p g (+ xp (* 5 x1)) (- ymax (* 5 y1)) 
                                           (+ xp (* 5 x2)) (- ymax (* 5 y2)))) ))  
                      
            ( draw (lambda(pix gc xs)
                    (set! from (car xs))
                    (for-each 
                        (lambda (x)
                            (drL pix gc from x)
                            (set! from x)) xs) ) ) 
                                 
            (win (gtk-window-new 0)) ; 0 means TOP WINDOW
                              
            (startit (gtk-frame-new))
            (endit   (gtk-frame-new))
            (kw "")
               
            ;; Create widget to display image
            (image (gtk-image-new)) 
               
            ;; Create eventbox 
	    (EBOX (gtk-event-box-new))
	    (_ (gtk-container-add EBOX image))
               
            (xsize 400)
	    (ysize 400)
               
            (rx 0)
            (ry 0)
            (nnn 2)
               
            (ext  (gtk-button-new-with-label "Exit"))
            (neg  (gtk-button-new-with-label "Neg"))
	    (xGt  (gtk-button-new-with-label "xGreater"))
	    (yGt  (gtk-button-new-with-label "yGreater"))
	                   
	    (hbx (gtk-hbox-new #f 0))
	    (vbx (gtk-vbox-new #f 2))
            (sep  (gtk-hseparator-new))
            (_ (pack-homogeneously hbx ext neg yGt xGt))
            (_ (pack-pack vbx hbx sep EBOX))
	    (_ (gtk-container-add win vbx ))    
            (_(gtk-widget-show-all win)  )
            (gdkwin (gtk-widget-get-parent-window image)) 
               
	    (PIX (gdk-pixmap-new gdkwin xsize ysize -1))
	    (GC (gdk-gc-new PIX))
	    (_ (gtk-image-set-from-pixmap image PIX))
	
            (COLOR (gtk-frame-new)))
	 	   
	   (gtk-window-set-resizable win #t)
	   (gtk-window-set-default-size win 600 400)
	   (gtk-window-set-position win 1)
	   
	   (gtk-window-set-title win "Homage to Escher/Delahaye")
                     
           (gdk-color-parse "#00ff00" COLOR)
           (gdk-gc-set-rgb-fg-color GC COLOR)
           
           (for-each  (lambda(x)
                           (draw PIX GC x)) cheval) 
           (gdk-draw-rectangle PIX GC #f 70 50 70 50)
           
           (play-with event until (equal? event ext) thus
                [ neg (set! nnn (- nnn)) ]
	        [ xGt  (set! rx (+ rx nnn))
	               (g-object-unref PIX)
	               (set! PIX (gdk-pixmap-new gdkwin xsize ysize -1))
	               (gtk-image-set-from-pixmap image PIX)
	               (gdk-color-parse "#00ff00" COLOR)
                       (gdk-gc-set-rgb-fg-color GC COLOR)
                       	               
	               (for-each (lambda(x)
		                    (set! ymax 400)  (set! xp 0)
		                    (draw PIX GC x)
		                    (set! ymax (- 400 ry))
		                    (draw PIX GC x)
		                    ;; ======================
		                    (set! ymax 400) (set! xp rx)
			            (draw PIX GC x)
				    (set! ymax (- 400 ry)) (set! xp rx)
		                    (draw PIX GC x)
                              ) cheval) ]
	        [ yGt   (set! ry (+ ry nnn))
	                (g-object-unref PIX)
                        (set! PIX (gdk-pixmap-new gdkwin xsize ysize -1))
                        (gtk-image-set-from-pixmap image PIX)
                        (gdk-color-parse "#ffffff" COLOR)
                        (gdk-gc-set-rgb-fg-color GC COLOR)
                        
                        (for-each 
                           (lambda(x)
                              (set! ymax 400)  (set! xp 0)
		              (draw PIX GC x)
		              (set! ymax (- 400 ry)) (set! xp 0)
		              (draw PIX GC x)
                              (set! ymax 400) (set! xp rx)
                              (draw PIX GC x)
                              (set! ymax (- 400 ry)) (set! xp rx)
                              (draw PIX GC x)
                              ) cheval)]   
             );close play-with   
       );close let*
); close define 


      
(define cheval '(
     ((10 10) (8 12) (9 16) (12 17) (13 18) (14 20)) 
     ((13 18) (12 19) (9 21) (9 20) (10 19) (9 17) (7 20) (8 22) (12 22))
     ((12 20) (12 22) (13 20) (16 31) (18 31) (19 32) )
     ( (16 31) (14 31 ) (14 32) ) 
    
     ( (14 31)(10 30) (12 31) (10 32)  (10 34)(11 34)(11 33)(10 33) )
    
     ( (12 32) (13 31)   )
     
     ( (10 34) (16 36) ) 
     ((16 35)(16 37)(18 35)(17 34)) ; ear
     ( (17 36) (20 36) (22 32) (19 26) ) ; behind the ear
     
     ( (20 36)(22 36)(22 34)  (24 32)(24 30)(19 26)  (18 23)(21 22)(21 24) ) ;back
     ( (30 30)(34 31)(36 31)(33 26)(32 22)  (28 22)(27 20)(29 17)(30 19)(29 20) ) ; wing
     ( (29 21)(32 19)(33 18) (32 17)(29 16)(28 12) (30 10)(21 4)(21 2) ) ;hinder legs
     ( (18 3)(19 6)(24 10)(24 12)  (22 14)(22 16)(23 17) );belly
     ( (22 16)(17 16)(16 17)(17 18) )
     
     ( (16 17)(16 16)(10 14)  (10 12)(12 11)(10 10));front leg
     
     ( (21 21)(22 24)(30 30) ); upper wing
     ( (24 24) (34 28))  ((25 23)(33 26)) ((25 21)(27 20));feathers
     ((23 21)(24 19))  ((27 20)(22 19)(22 21)) ((22 19)(21 20))
     
     ((13 34)(15 35)(16 34)(16 33))
     ((15 35)(15 34)(16 34)(15 34)(15 35)); eye
     ((24 12)(26 10)(19 5)(19 3)) ((28 22)(25 22)) 
) )
