;;
;; Compile with:
;;
;;	bigloo -o scr-ide scr-ide.scm gtkserver.o -copt -w -lgtk-server
;;

;; This program shows how to write a simple GUI for Bigloo.
;; It was kept simple, so you can understand it easily, and
;; add features of your own.
;;
;; 1- Type the source code on the text viewer. 
;;
;; 2- If you want to save your source code, write the name of the
;;    file on the main text entry, and press the button "Save". 
;;
;; 3- To load a file, write the file name on the main text entry, and 
;;    press "Load". 
;;
;; 4- If you want to browse your computer, in order to find a good 
;;    place to save your source file, press the button "Browse". 
;;    You can use the browsing widget to inspect the file tree of your
;;    computer. Type the path on the browser's text entry, and press "OK".
;;    The browser will be closed, and the path will be transfered to 
;;    the main data entry.
;; 5- To search a string, write the string that you want to search on
;;    the main text entry, and press search. If you want to find the
;;    next instance of the string, press NxtSrc.
;; 
;; 6- To check parentheses, select the expression, and press CheckPar.
;;    If there is unclosed parenthesis, the expression will become red.
;;    Correct the error, select the expression again, and press CheckPar.
;;    After the correction, the selected expression will become black again.
;; 7- You can generate a Latex documentation of your program. To this
;;    end, it must start with comments, that will form the LaTeX introduction.
;;    A double semi-colon introduces a \section. Take a look at que.scx
;;    to see an example of source that you can latexify. The button
;;    "Set Title" reads the documentation title from the main text entry.
;;    The button "To LaTeX" taks a file name from the main text entry,
;;    add the extension ".tex" to it, and generates a LaTeX documentation.

(module test (main main)
  (include "macro-def.sch")
  (import (gtkserver_gtk "gtk.scm"))
  (eval (export-module)))

(define (main arqv)
  (gtk-init)
  (let* ( (win::string (gtk-window-new 0))
          (win2::string (gtk-window-new 0))
          (_ (gtk-window-set-default-size win 800 600))
          (_ (gtk-window-set-default-size win2 600 400))
          (fileWin (gtk-window-new 0)) 
          (_ (gtk-window-set-default-size fileWin 400 200))
          (fileTable (gtk-table-new 10 20 0))
          (tbl::string  (gtk-table-new 20 20 1))
          (tb2  (gtk-table-new 10 10 1))
          (fromit (gtk-frame-new))
          (startit (gtk-frame-new))
          (endit   (gtk-frame-new))
          (fntstart (gtk-frame-new))
          (fntend (gtk-frame-new))
          (mrk #f)
          
          (sav::string  (gtk-button-new-with-label "Save"))
          (lod::string  (gtk-button-new-with-label "Load"))
          
          (sav2::string  (gtk-button-new-with-label "Save"))
          (lod2::string  (gtk-button-new-with-label "Load"))
          
          (ok  (gtk-button-new-with-label "Ok"))
          
          (src (gtk-button-new-with-label "Search"))
          (nsr (gtk-button-new-with-label "NxtSrc"))
          (par (gtk-button-new-with-label "((()))"))
          (pretty (gtk-button-new-with-label "Pretty"))
          (sTt        (gtk-button-new-with-label "Title"))
          (ltx::string (gtk-button-new-with-label "LaTeX"))
          (hide-button  (gtk-button-new-with-label "Hide"))
          
          (enFile (gtk-entry-new))
          (filsel (gtk-file-chooser-widget-new 0))
          (fileFilter (gtk-file-filter-new))
          (hfx (gtk-hbox-new #t 10))
         
          (en (gtk-entry-new))
          (en2 (gtk-entry-new))
          
          (hbx::string (gtk-hbox-new #t 4))
          (hb2 (gtk-hbox-new #t 4))
          
          (hby::string (gtk-hbox-new #t 4))
          (evl (gtk-button-new-with-label "eval"))
          
          (mnu-bar  (gtk-menu-bar-new))
          (filemenu (gtk-menu-new))
          (open-file (gtk-menu-item-new-with-label "Browse"))
          (data-entry-scheme (gtk-menu-item-new-with-label "Data-Entry"))
          (new-buffer  (gtk-menu-item-new-with-label "New Buffer"))
          (sep3 (gtk-menu-item-new))
          (ext  (gtk-menu-item-new-with-label "Exit")) 
          
          
          (tit1 (gtk-menu-item-new-with-mnemonic "_File"))
          
          (vbx::string (gtk-vbox-new #f 2))
          (vb2::string (gtk-vbox-new #f 2))
          
          (sep  (gtk-hseparator-new)) 
          (sw (gtk-scrolled-window-new))
          (txt (gtk-text-buffer-new))
          
          (sw2 (gtk-scrolled-window-new))
          (tx2 (gtk-text-buffer-new))
          (tv2 (gtk-text-view-new-with-buffer tx2))
          
          (tv (gtk-text-view-new-with-buffer txt))
          (docTitle "Program documentation")
          (fname "a") 
          (browseName "c:/")
          (search-string "")
          (fnt-nm "arial 12")
          (fntname "arialfont")
          (kw "")
          (fnd #f) 
          (buf '()))
     
     (gtk-menu-shell-append filemenu open-file)
     (gtk-menu-shell-append filemenu new-buffer)
     (gtk-menu-shell-append filemenu data-entry-scheme)
     (gtk-menu-shell-append filemenu sep3)
     (gtk-menu-shell-append filemenu ext)
     
     (gtk-menu-item-set-submenu tit1 filemenu)
     (gtk-menu-shell-append mnu-bar tit1)
     
     (pack-homogeneously hbx mnu-bar lod sav evl src nsr par pretty sTt ltx)
     (pack-homogeneously  hb2  hide-button lod2)
     
       
     (pack-pack vbx  hbx en sep) ;en sep)
     (pack-pack vb2  hb2 en2)
     (gtk-text-view-set-editable tv2 1)
     (gtk-text-view-set-wrap-mode tv2 2)
     (gtk-container-add sw2 tv2)
     (gtk-table-attach-defaults tb2 vb2 1 9 0 1)
     (gtk-container-add  win2 tb2)
     (gtk-table-attach-defaults tb2 sw2 1 9 2 9)
      
     (gtk-container-add sw tv)
     ;(gtk-table-attach-defaults tbl mnu-bar 1 4 0 1)
     (gtk-table-attach-defaults tbl vbx 1 19 0 2 )
     ;(gtk-table-attach-defaults tbl en 1 19 2 4)
     (gtk-table-attach-defaults tbl sw 1 20 3 19)
	   
     (gtk-text-view-set-editable tv 1)
     (gtk-text-view-set-wrap-mode tv 2)
	  
     (gtk-container-add  win tbl)
     (gtk-window-set-title win "Simple IDE")  
      
     (gtk-widget-show-all win)
     
     (gtk-box-pack-start hfx enFile 0 0 1)
     (gtk-box-pack-start hfx ok 0 0 1) 
     (gtk-table-attach-defaults fileTable hfx 0 10 0 1)
     (gtk-table-attach-defaults fileTable filsel 0 19 2 10)
     (gtk-container-add fileWin fileTable)
     
     (gtk-file-filter-set-name fileFilter  "all")
     (gtk-file-filter-add-pattern fileFilter "*.*")
     (gtk-file-chooser-add-filter filsel fileFilter)
     
     (gtk-text-buffer-create-tag txt "fstcolor" "foreground" "red")
     
     (gtk-text-buffer-create-tag txt fntname "font" fnt-nm)
     
     (do ((event (gtk-server-callback) 
	         (gtk-server-callback)) )
        ((equal? event ext)) 
        (cond ( (equal? event sav)
                
                (set! fname (gtk-entry-get-text en))
                                      
                (set! fname (if (< (string-length fname) 2) 
                                    "backup.txt" fname))
                (gtk-save-buffer txt fname startit endit)
                (gtk-text-buffer-get-bounds txt fntstart fntend)
	        (gtk-text-buffer-apply-tag-by-name txt fntname fntstart fntend) )
	        
	      ( (equal? event lod)
                (set! fname (gtk-entry-get-text en))
                (gtk-text-buffer-set-text  txt "" -1)
		(let again ( (xs (gtk-load-lines fname)))
		      (when (not (null? xs))
                          (gtk-text-buffer-get-end-iter txt endit)
                          (gtk-text-buffer-insert txt endit  (car xs)  -1) 
		          (again (cdr xs))))
		(gtk-text-buffer-get-bounds txt fntstart fntend)
	        (gtk-text-buffer-apply-tag-by-name txt fntname fntstart fntend))
	      ( (equal? event lod2)
	                      (set! fname (gtk-entry-get-text en2))
	                      (gtk-text-buffer-set-text  tx2 "" -1)
	      		(let again ( (xs (gtk-load-lines fname)))
	      		      (when (not (null? xs))
	                                (gtk-text-buffer-get-end-iter tx2 endit)
	                                (gtk-text-buffer-insert tx2 endit  (car xs)  -1) 
	      		          (again (cdr xs)))) )
              ( (equal? event sTt)
                (set! docTitle (gtk-entry-get-text en))
               
                (when (< (string-length docTitle) 2)
                      (set! docTitle "Program Documentation")))
              ( (equal? event ltx)
	        (set! fname (gtk-entry-get-text en))
	                             
	        (set! fname (if (< (string-length fname) 2) 
	                                          "backup" fname))
	      	(gtk-text-buffer-get-bounds txt startit endit)
	      	(set! kw (gtk-text-buffer-get txt startit endit #t) )
	        (latexIt kw (string-append fname ".tex") docTitle))
              ( (equal? event ok)
                (set! fname (gtk-entry-get-text enFile))
                
                (set!  browseName (gtk-file-chooser-get-filename filsel))
                   
                (string-replace! browseName #\\ #\/)
                
                (print browseName) (flush-output-port (current-output-port))
                
                (gtk-entry-set-text en (string-append browseName fname))
                                 
                (gtk-widget-hide fileWin))
              ( (equal? event open-file) ;brs)
                (gtk-widget-show-all fileWin) 
                (gtk-file-chooser-set-filename filsel browseName) )
	      ( (equal? event src) (first-search en txt fromit startit endit))
	      ( (and (equal? event nsr) mrk) (next-search en txt mrk fromit startit endit))
	      ( (equal? event evl)
	      	      	        (set! fnd (gtk-text-buffer-get-selection-bounds txt startit endit))
	      	      	        (cond (   fnd 
	      	      	                 (set! kw (gtk-text-buffer-get-text txt startit endit #t))
	      	      	                 (gtk-text-buffer-remove-tag-by-name txt "fstcolor" startit endit)
	      	      	                 (cond ( (check-parentheses kw)
	      	      	                        
	      	      	                         
	      	      	                         (set! kw (expression->string 
	      	      	                              (with-handler 
	      	      	                                    (lambda(exc) exc)
	      	      	                                 (eval (string->expression kw)))))
	      	      	                         (gtk-entry-set-text en kw)
	      	      	                         
	      	      	                         
	      	      	                       )
	      	      	                       ( else (gtk-entry-set-text en "Something is rotten!")
	      	      	                              (gtk-text-buffer-apply-tag-by-name txt "fstcolor" startit endit) 
	      	      	                       )
	      	      	                 );close inner cond
	      	      	             );close selection found for checking parentheses
	      	      	             (  else ;no-selection for checking parentheses.
	      	      	                (gtk-entry-set-text en "Please, select the text you want to eval")
	      	      	             );close selection not made condition 
	      	      	       );close selection cond
	      );close check parentheses condition
	      ( (equal? event data-entry-scheme)
	        (set! fname "moda.ptp")
		(gtk-text-buffer-set-text  txt "" -1)
                (let again ( (xs (gtk-load-lines fname)))
                       (when (not (null? xs))
                           (gtk-text-buffer-get-end-iter txt endit)
                           (gtk-text-buffer-insert txt endit  (car xs)  -1) 
                           (again (cdr xs))))
                (gtk-text-buffer-get-bounds txt fntstart fntend)
                (gtk-text-buffer-apply-tag-by-name txt fntname fntstart fntend)
	        
	      )
	      ( (equal? event new-buffer)
	        (gtk-widget-show-all win2)
	      )
	      ( (equal? event hide-button)
	        (gtk-widget-hide win2)
	      )
	      ( (equal? event pretty)
	      	        (set! fnd (gtk-text-buffer-get-selection-bounds txt startit endit))
	      	        (cond (   fnd 
	      	                 (set! kw (gtk-text-buffer-get-text txt startit endit #t))
	      	                 (gtk-text-buffer-remove-tag-by-name txt "fstcolor" startit endit)
	      	                 (cond ( (check-parentheses kw)
	      	                        
	      	                         (gtk-entry-set-text en "That is great!")
	      	                        
	      	                         (set! kw (expression->pretty (string->expression kw)))
	      	                         (gtk-text-buffer-delete txt startit endit)
	      	                         (gtk-text-buffer-insert txt endit kw  -1)  
	      	                       )
	      	                       ( else (gtk-entry-set-text en "Something is rotten!")
	      	                              (gtk-text-buffer-apply-tag-by-name txt "fstcolor" startit endit) 
	      	                       )
	      	                 );close inner cond
	      	             );close selection found for checking parentheses
	      	             (  else ;no-selection for checking parentheses.
	      	                (gtk-entry-set-text en "Please, select the text you want to check")
	      	             );close selection not made condition 
	      	       );close selection cond
	      );close check parentheses condition
	      ( (equal? event par)
	        (set! fnd (gtk-text-buffer-get-selection-bounds txt startit endit))
	        (cond (   fnd 
	                 (set! kw (gtk-text-buffer-get-text txt startit endit #t))
	                 (gtk-text-buffer-remove-tag-by-name txt "fstcolor" startit endit)
	                 (cond ( (check-parentheses kw)
	                        
	                         (gtk-entry-set-text en "That is great!")
	                        
	                         
	                       )
	                       ( else (gtk-entry-set-text en "Something is rotten!")
	                              (gtk-text-buffer-apply-tag-by-name txt "fstcolor" startit endit) 
	                       )
	                 );close inner cond
	             );close selection found for checking parentheses
	             (  else ;no-selection for checking parentheses.
	                (gtk-entry-set-text en "Please, select the text you want to check")
	             );close selection not made condition 
	       );close selection cond
	    );close check parentheses condition
	);close outer cond
	        
      );close do 
   );close let*
);close define

;;(p2  (open-input-string normal-string)))


(define (latexIt txt file-name docTitle)
       (let* ((pin (open-input-c-string txt))
              (cstring (read pin)))
        (with-input-from-string cstring
          (lambda()
             (let ( (resp '()))
	           (do ( (i (read-line) (read-line)))
	               ( (eof-object? i) (save-latex (wrap-document docTitle (latexify resp)) file-name))
	               (set! resp (cons i  resp))
	           )
	     )
	  )
	 )
	)
 )
  

(define (check-parentheses s)
   (let ( (zero-sum 0)
          (sz (- (string-length s) 1)  ) )
       (do ( (i 0 (+ i 1)))
           ( (> i sz) (equal? zero-sum 0))
           (when (equal? (string-ref s i) #\( )
                 (set! zero-sum (+ zero-sum 1)))
           (when (equal? (string-ref s i) #\) )
                 (set! zero-sum (- zero-sum 1))) ) ) )

(define (latexify xs)
  (app (get-doc #t xs '() '())))
  
(define (app xs)
   (cond ( (null? xs) '())
         (else (append (car xs) '(" ") (app (cdr xs)) )) ))

         
(define (latex-doc xs acc)
   (cond ( (null? xs) acc)
         (else (latex-doc (cdr xs) (cons (remove-comment (car xs)) acc) )
         )
   )
)

(define (is-comment? x)
   (cond ( (< (string-length x) 1) #f)
         ( (equal? (string-ref x 0) #\;) #t)
         (else #f)))
         
(define (not-comment? x)
   (cond ( (< (string-length x) 1) #t)
         ( (equal? (string-ref x 0) #\;) #f )
         (else #t)))

(define (add-block xs block)
  (cond ( (null? xs) block)
        (else (cons xs block))))

(define (get-doc comment xs acc blocks)
  (cond ( (and comment (null? xs)) (add-block acc blocks))
        ( (and (not comment) (null? xs)) (add-block (wrap-verbatim acc) blocks))
        ( (and comment (is-comment? (car xs))) 
          (get-doc comment (cdr xs) (cons  (remove-comment (car xs)) acc) blocks))
          
        ( (and comment (not-comment? (car xs)))
          (get-doc (not comment) (cdr xs) (list (car xs)) (add-block acc blocks)))
        ( (and (not comment) (not-comment? (car xs)))
          (get-doc comment (cdr xs) (add-block (car xs) acc) blocks))
        ( (and (not comment) (is-comment? (car xs)))
          (get-doc (not comment) (cdr xs) (list (remove-comment (car xs))) (cons (wrap-verbatim acc) blocks)))))


(define (remove-comment x)
   (cond ( (< (string-length x) 1) x)
         ( (section? x) (wrap-section x))
         ( (equal? (string-ref x 0) #\;) (string-set! x 0 #\space) x) 
         (else x) 
   )
)

(define (section? x)
   (if (< (string-length x) 4) #f
       (and (equal? (string-ref x 0) #\;) (equal? (string-ref x 1) #\;)) ) )
       
(define (wrap-section x)
  (string-set! x 0 #\space)
  (string-set! x 1 #\space)
  (string-append "\\section{" x "}"))
      
(define (save-latex xs file-name)
  (call-with-output-file file-name 
     (lambda(out)
        (for-each (lambda(x) (display x out) (newline out)) xs)
        (close-output-port out)
     )     
  )
)

(define (wrap-verbatim xs)
   (append '("\\begin{verbatim}") xs '("\\end{verbatim}") ))

(define (wrap-document docTitle xs)
  (append (latex-header docTitle) xs '("\\end{document}")))

(define (latex-header title)
 (list "\\documentclass[12pt]{article}"
   (string-append "\\title{" title "}")
   "\\author{Ed Mort}"
   "\\date{}"
   "\\begin{document}"
   "\\maketitle"
))

